==> snapstraction-0.3.5a1-py2-none-any.whl: snapstraction/nvparam.py <==
"""SNAPstraction NVParam ops"""

from collections import OrderedDict

from tornado.gen import coroutine, Return

import formatters
from .util import first_or_none, fix_up_target


nvparam_lookup = OrderedDict([
    ('net.network-id', {'id': 3, 'formatter': formatters.format_as_hex}),
    ('net.channel', {'id': 4}),
    ('net.radio-unicast-retries', {'id': 19}),
    ('net.radio-lq-threshold', {'id': 39}),
    ('net.default-radio-rate', {'id': 90}),

    ('device.mac-address', {'id': 2, 'formatter': formatters.format_as_mac}),
    ('device.mfg-date', {'id': 7}),
    ('device.name', {'id': 8}),
    ('device.last-error', {'id': 9}),
    ('device.type', {'id': 10}),
    ('device.feature-bits', {'id': 11, 'formatter': formatters.format_as_hex, 'bitfields': OrderedDict([
        ('enable-uart0', 0),
        ('enable-flow-control-uart0', 1),
        ('enable-uart1', 2),
        ('enable-flow-control-uart1', 3),
        ('enable-power-amplifier', 4),
        ('enable-external-power-down-output', 5),
        ('enable-alternate-clock-source', 6),
        ('enable-ds-audio', 7),
        ('enable-second-crc', 8),
        ('reduce-tx-power-levels', 9),
        ('enable-packet-crc', 10),
        ('enable-big-route-table', 11),
    ])}),
    ('device.snappy-crc', {'id': 40, 'formatter': formatters.format_as_hex}),
    ('device.platform', {'id': 41}),
    ('device.alternate-radio-trim', {'id': 63}),
    ('device.vendor-config-bits', {'id': 64, 'formatter': formatters.format_as_hex}),

    ('mcast.processed-groups', {'id': 5}),
    ('mcast.forwarded-groups', {'id': 6}),
    ('mcast.carrier-sense', {'id': 16}),
    ('mcast.collision-detection', {'id': 17}),
    ('mcast.collision-avoidance', {'id': 18}),
    ('mcast.cs-cd-threshold', {'id': 33}),
    ('mcast.serial-forwarded-groups', {'id': 78}),
    ('mcast.csma-settings', {'id': 91}),

    ('uart.default', {'id': 12}),
    ('uart.buffering-timeout', {'id': 13}),
    ('uart.buffering-threshold', {'id': 14}),
    ('uart.intercharacter-timeout', {'id': 15}),
    ('uart.uart0-default-baudrate', {'id': 80}),
    ('uart.uart1-default-baudrate', {'id': 81}),

    ('mesh.routing.max-timeout', {'id': 20}),
    ('mesh.routing.min-timeout', {'id': 21}),
    ('mesh.routing.new-timeout', {'id': 22}),
    ('mesh.routing.used-timeout', {'id': 23}),
    ('mesh.routing.delete-timeout', {'id': 24}),
    ('mesh.routing.rreq-retries', {'id': 25}),
    ('mesh.routing.rreq-wait-timeout', {'id': 26}),
    ('mesh.routing.initial-hop-limit', {'id': 27}),
    ('mesh.routing.max-hop-limit', {'id': 28}),
    ('mesh.seq-number', {'id': 29}),
    ('mesh.override', {'id': 30, 'formatter': formatters.format_as_boolean}),
    ('mesh.penalty-lq-threshold', {'id': 31}),
    ('mesh.rejection-lq-threshold', {'id': 32}),

    ('security.enable-encryption', {'id': 50, 'formatter': formatters.format_as_encryption_status}),
    ('security.encryption-key', {'id': 51}),
    ('security.lockdown-bitmask', {'id': 52, 'formatter': formatters.format_as_boolean}),

    ('max-loyalty', {'id': 53}),
    ('remaining-reboots', {'id': 61}),
    ('clock-regulator', {'id': 65}),
    ('radio-calibration', {'id': 66}),
    ('tx-power-limit', {'id': 70}),
])

nvparam_reverse_lookup = {param_info['id']: param_name for param_name, param_info in nvparam_lookup.iteritems()}


@coroutine
def check_and_set_nvparam(gus, target, param_id, param_value, retries=3):
    """Ensure the NVParam has the specific value"""

    attempt = 0
    while attempt < retries:
        attempt += 1
        yield gus.callback_ucast_rpc(target, "saveNvParam", param_id, param_value)
        val = yield gus.callback_ucast_rpc(target, "loadNvParam", param_id)
        val = first_or_none(val)

        if isinstance(val, int):
            val = val & 0xFFFF

        if val == param_value:
            raise Return(val)

    raise Exception("Couldn't set NV Param %r to %r for node %r", param_id,
                    param_value, target)


def parse_nvparam(param):
    """Parse a string representing an NVParam into the NVParam name and id.

    The input string must either contain the integer string id or the parameter
    name, as defined in the nvparam_lookup dictionary.

    Args:
        param (str): The NVParam id or name

    Returns:
        dict{'param_id': int, 'bit_id': int, 'param_name': str, formatter: function)
    """
    param_id = None
    param_name = None
    bit_id = None
    formatter = None

    try:
        # Attempt to simply convert the name to an integer id
        param_id = int(param)

        try:
            # Perform a reverse dictionary lookup to get the param name from the id
            param_name = nvparam_reverse_lookup[param_id]
            return parse_nvparam(param_name)
        except KeyError:
            # The param with that id does not have a name, so use the integer id as the param name
            param_name = param
    except ValueError:
        try:
            # First, see if the nvparam is a bitfield
            for (param_name, param_info) in nvparam_lookup.iteritems():
                # The nvparam is a bitfield if it consists of a valid param name followed by a period
                if param.startswith(param_name + '.') and 'bitfields' in param_info:
                    # Remove the param_name and the period to get the bit name
                    bit_name = param[len(param_name) + 1:]
                    # Lookup the bit id from the bit name
                    bit_id = param_info['bitfields'][bit_name]
                    param_id = param_info['id']
                    formatter = formatters.format_as_boolean
                    break
            else:
                # The nvparam is not a bitfield, so assume that it is a simple param name
                param_info = nvparam_lookup[param]
                param_id = param_info['id']
                formatter = param_info.get('formatter', None)

            param_name = param
        except KeyError:
            raise ValueError('%s is not a valid NV param name' % param)

    return {
        'param_id': param_id,
        'bit_id': bit_id,
        'param_name': param_name,
        'formatter': formatters.format_noop if formatter is None else formatter
    }


@coroutine
@fix_up_target
def get_nvparam(gus, data, target, param):
    """Get the specified NVParam value from the target.

    Equivalent of `loadNvParam`

    Args:
        gus (Gus): Grand Unified SNAP instance
        data (OrderedDict()): Dictionary for result data
        target (str, SNAP Address): Target Node
        param (str): NVParam ID to get

    Returns:
        None

    """
    parsed_nvparam = parse_nvparam(param)
    (param_id, bit, param_name, formatter) = [parsed_nvparam.get(x) for x in ['param_id', 'bit_id', 'param_name', 'formatter']]

    val = yield gus.callback_ucast_rpc(target, "loadNvParam", param_id)
    param_val = first_or_none(val)

    if param_val is not None and bit is not None:
        # Extract the correct bit from the param_val
        param_val = param_val >> bit & 1

    data['nvparam %s' % param_name] = formatter(param_val)


@coroutine
@fix_up_target
def set_nvparam(gus, data, target, param, param_value):
    """Set the NVParam on the target to the value.

    Equivalent of `saveNvParam` plus a read-back to verify.

    Args:
        gus (Gus): Grand Unified SNAP instance
        data (OrderedDict()): Dictionary for result data
        target (str, SNAP Address): Target Node
        param (str): NVParam ID to set
        param_value: Value to write

    Returns:
        None

    """
    parsed_nvparam = parse_nvparam(param)
    (param_id, bit, param_name, formatter) = [parsed_nvparam.get(x) for x in ['param_id', 'bit_id', 'param_name', 'formatter']]

    if bit is not None:
        val = yield gus.callback_ucast_rpc(target, "loadNvParam", param_id)
        param_val = first_or_none(val)

        param_value = (param_val & ~(1 << bit)) | (int(param_value) << bit)

    val = yield check_and_set_nvparam(gus, target, param_id, param_value)
    param_val = first_or_none(val)

    if param_val is not None and bit is not None:
        # Extract the correct bit from the param_val
        param_val = param_val >> bit & 1

    data['nvparam %s' % param_name] = formatter(param_val)


@coroutine
@fix_up_target
def reset_nvparams(gus, data, target):
    """OTA NVParam reset to factory defaults.

    Resets the NVParams by calling saveNvParam(1, 0).
    Follow with a reboot to make them take effect.

    Args:
        gus (Gus): Grand Unified SNAP instance
        data (OrderedDict()): Dictionary for result data
        target (str, SNAP Address): Target Node

    Returns:
        None

    """
    val = yield gus.callback_ucast_rpc(target, "saveNvParam", 1, 0)
    data['reset nvparam'] = first_or_none(val)

==> snapstraction-0.8.1-py2-none-any.whl: snapstraction/nvparam.py <==
"""SNAPstraction NVParam ops"""

from collections import OrderedDict

from tornado.gen import coroutine, Return

import formatters
from .util import first_or_none, fix_up_target


nvparam_lookup = OrderedDict([
    ('net.network-id', {'id': 3, 'formatter': formatters.format_as_hex}),
    ('net.channel', {'id': 4}),
    ('net.radio-unicast-retries', {'id': 19}),
    ('net.radio-lq-threshold', {'id': 39}),
    ('net.default-radio-rate', {'id': 90}),

    ('device.mac-address', {'id': 2, 'formatter': formatters.format_as_mac}),
    ('device.mfg-date', {'id': 7}),
    ('device.name', {'id': 8}),
    ('device.last-error', {'id': 9}),
    ('device.type', {'id': 10}),
    ('device.feature-bits', {'id': 11, 'formatter': formatters.format_as_hex, 'bitfields': OrderedDict([
        ('enable-uart0', 0),
        ('enable-flow-control-uart0', 1),
        ('enable-uart1', 2),
        ('enable-flow-control-uart1', 3),
        ('enable-power-amplifier', 4),
        ('enable-external-power-down-output', 5),
        ('enable-alternate-clock-source', 6),
        ('enable-ds-audio', 7),
        ('enable-second-crc', 8),
        ('reduce-tx-power-levels', 9),
        ('enable-packet-crc', 10),
        ('enable-big-route-table', 11),
    ])}),
    ('device.snappy-crc', {'id': 40, 'formatter': formatters.format_as_hex}),
    ('device.platform', {'id': 41}),
    ('device.alternate-radio-trim', {'id': 63}),
    ('device.vendor-config-bits', {'id': 64, 'formatter': formatters.format_as_hex}),

    ('mcast.processed-groups', {'id': 5, 'formatter': formatters.format_as_hex}),
    ('mcast.forwarded-groups', {'id': 6, 'formatter': formatters.format_as_hex}),
    ('mcast.carrier-sense', {'id': 16, 'formatter': formatters.format_as_boolean}),
    ('mcast.collision-detection', {'id': 17, 'formatter': formatters.format_as_boolean}),
    ('mcast.collision-avoidance', {'id': 18, 'formatter': formatters.format_as_boolean}),
    ('mcast.cs-cd-threshold', {'id': 33}),
    ('mcast.serial-forwarded-groups', {'id': 78, 'formatter': formatters.format_as_hex}),
    ('mcast.csma-settings', {'id': 91}),

    ('uart.default', {'id': 12, 'formatter': formatters.format_as_unsigned}),
    ('uart.buffering-timeout', {'id': 13, 'formatter': formatters.format_as_unsigned}),
    ('uart.buffering-threshold', {'id': 14}),
    ('uart.intercharacter-timeout', {'id': 15}),
    ('uart.uart0-default-baudrate', {'id': 80}),
    ('uart.uart1-default-baudrate', {'id': 81}),

    ('mesh.routing.max-timeout', {'id': 20, 'formatter': formatters.format_as_unsigned}),
    ('mesh.routing.min-timeout', {'id': 21, 'formatter': formatters.format_as_unsigned}),
    ('mesh.routing.new-timeout', {'id': 22, 'formatter': formatters.format_as_unsigned}),
    ('mesh.routing.used-timeout', {'id': 23, 'formatter': formatters.format_as_unsigned}),
    ('mesh.routing.delete-timeout', {'id': 24, 'formatter': formatters.format_as_unsigned}),
    ('mesh.routing.rreq-retries', {'id': 25, 'formatter': formatters.format_as_unsigned}),
    ('mesh.routing.rreq-wait-timeout', {'id': 26, 'formatter': formatters.format_as_unsigned}),
    ('mesh.routing.initial-hop-limit', {'id': 27, 'formatter': formatters.format_as_unsigned}),
    ('mesh.routing.max-hop-limit', {'id': 28, 'formatter': formatters.format_as_unsigned}),
    ('mesh.seq-number', {'id': 29, 'formatter': formatters.format_as_unsigned}),
    ('mesh.override', {'id': 30, 'formatter': formatters.format_as_boolean}),
    ('mesh.penalty-lq-threshold', {'id': 31}),
    ('mesh.rejection-lq-threshold', {'id': 32}),

    ('security.enable-encryption', {'id': 50, 'formatter': formatters.format_as_encryption_status}),
    ('security.encryption-key', {'id': 51}),
    ('security.lockdown-bitmask', {'id': 52, 'formatter': formatters.format_as_hex}),

    ('max-loyalty', {'id': 53}),
    ('remaining-reboots', {'id': 61, 'formatter': formatters.format_as_unsigned}),
    ('clock-regulator', {'id': 65}),
    ('radio-calibration', {'id': 66}),
    ('tx-power-limit', {'id': 70}),
])

nvparam_reverse_lookup = {param_info['id']: param_name for param_name, param_info in nvparam_lookup.iteritems()}


@coroutine
def check_and_set_nvparam(gus, target, param_id, param_value, retries=3):
    """Ensure the NVParam has the specific value"""

    attempt = 0
    while attempt < retries:
        attempt += 1
        yield gus.callback_ucast_rpc(target, "saveNvParam", param_id, param_value)
        val = yield gus.callback_ucast_rpc(target, "loadNvParam", param_id)
        val = first_or_none(val)

        if isinstance(val, int):
            val = val & 0xFFFF

        if val == param_value:
            raise Return(val)

    raise Exception("Couldn't set NV Param %r to %r for node %r", param_id,
                    param_value, target)


def parse_nvparam(param):
    """Parse a string representing an NVParam into the NVParam name and id.

    The input string must either contain the integer string id or the parameter
    name, as defined in the nvparam_lookup dictionary.

    Args:
        param (str): The NVParam id or name

    Returns:
        dict{'param_id': int, 'bit_id': int, 'param_name': str, formatter: function)
    """
    param_id = None
    param_name = None
    bit_id = None
    formatter = None

    try:
        # Attempt to simply convert the name to an integer id
        param_id = int(param)

        try:
            # Perform a reverse dictionary lookup to get the param name from the id
            param_name = nvparam_reverse_lookup[param_id]
            return parse_nvparam(param_name)
        except KeyError:
            # The param with that id does not have a name, so use the integer id as the param name
            param_name = param
    except ValueError:
        try:
            # First, see if the nvparam is a bitfield
            for (param_name, param_info) in nvparam_lookup.iteritems():
                # The nvparam is a bitfield if it consists of a valid param name followed by a period
                if param.startswith(param_name + '.') and 'bitfields' in param_info:
                    # Remove the param_name and the period to get the bit name
                    bit_name = param[len(param_name) + 1:]
                    # Lookup the bit id from the bit name
                    bit_id = param_info['bitfields'][bit_name]
                    param_id = param_info['id']
                    formatter = formatters.format_as_boolean
                    break
            else:
                # The nvparam is not a bitfield, so assume that it is a simple param name
                param_info = nvparam_lookup[param]
                param_id = param_info['id']
                formatter = param_info.get('formatter', None)

            param_name = param
        except KeyError:
            raise ValueError('%s is not a valid NV param name' % param)

    return {
        'param_id': param_id,
        'bit_id': bit_id,
        'param_name': param_name,
        'formatter': formatters.format_noop if formatter is None else formatter
    }


@coroutine
@fix_up_target
def get_nvparam(gus, target, param):
    """Get the specified NVParam value from the target.

    Equivalent of `loadNvParam`

    Args:
        gus (Gus): Grand Unified SNAP instance
        target (str, SNAP Address): Target Node
        param (str): NVParam ID to get

    Returns:
        A dict containing the NVParam data

    """
    data = {}
    parsed_nvparam = parse_nvparam(param)
    (param_id, bit, param_name, formatter) = [parsed_nvparam.get(x) for x in ['param_id', 'bit_id', 'param_name', 'formatter']]

    val = yield gus.callback_ucast_rpc(target, "loadNvParam", param_id)
    param_val = first_or_none(val)

    if param_val is not None and bit is not None:
        # Extract the correct bit from the param_val
        param_val = param_val >> bit & 1

    data['nvparam %s' % param_name] = formatter(param_val)

    raise Return(data)


@coroutine
@fix_up_target
def set_nvparam(gus, target, param, param_value):
    """Set the NVParam on the target to the value.

    Equivalent of `saveNvParam` plus a read-back to verify.

    Args:
        gus (Gus): Grand Unified SNAP instance
        target (str, SNAP Address): Target Node
        param (str): NVParam ID to set
        param_value: Value to write

    Returns:
        A dict containing the NVParam data

    """
    data = {}
    parsed_nvparam = parse_nvparam(param)
    (param_id, bit, param_name, formatter) = [parsed_nvparam.get(x) for x in ['param_id', 'bit_id', 'param_name', 'formatter']]

    if bit is not None:
        val = yield gus.callback_ucast_rpc(target, "loadNvParam", param_id)
        param_val = first_or_none(val)

        param_value = (param_val & ~(1 << bit)) | (int(param_value) << bit)

    val = yield check_and_set_nvparam(gus, target, param_id, param_value)
    param_val = first_or_none(val)

    if param_val is not None and bit is not None:
        # Extract the correct bit from the param_val
        param_val = param_val >> bit & 1

    data['nvparam %s' % param_name] = formatter(param_val)
    raise Return(data)


@coroutine
@fix_up_target
def reset_nvparams(gus, target):
    """OTA NVParam reset to factory defaults.

    Resets the NVParams by calling saveNvParam(1, 0).
    Follow with a reboot to make them take effect.

    Args:
        gus (Gus): Grand Unified SNAP instance
        target (str, SNAP Address): Target Node

    Returns:
        A dict containing the reset response

    """
    data = {}
    val = yield gus.callback_ucast_rpc(target, "saveNvParam", 1, 0)
    data['reset nvparam'] = first_or_none(val)
    raise Return(data)
